
/** The Dispatcher receives messages from Messengers 
    and forwards them onto all other Messengers. */
public class Dispatcher {
	
	private Messenger[] messengers;
	int verbosity = 7;	// range of 0 to 10 for nothing to lots of information
	
	public Dispatcher() {
		// Create the messengers
		messengers = new Messenger[] {
			new EnchantingMessenger("Enchanting Messenger", this, 200, verbosity, ScratchIO.ENCHANTING_PORT),
			new EnchantingMessenger("Scratch Messenger", this, 4000, verbosity, ScratchIO.SCRATCH_PORT),
			new NXTMessenger("NXT Messenger", this, 5000, verbosity),
		};
	}
	
	/** Takes a message and forwards it on to other interested parties. */
	public void forward(ScratchMessage msg, Messenger source) {
		if (verbosity > 6) {
			//System.out.println("");
			System.out.println("" + source + " asks for dispatch of: " + msg.getFullMessage() );
		}
		for (int i = 0; i < messengers.length; ++i) {
			if (source != messengers[i]) {
				boolean success = messengers[i].sendMessage(msg, source);
				
				if (verbosity > 8) {
					System.out.println("Sending to " + messengers[i] + 
						(success ? " succeeded." : " failed."));				
				}
			}
		}
	}
	
	/** Forward a message to a specific part */
	public void forwardTo(ScratchMessage msg, Messenger source, String destinationName) {
		boolean sent = false;
		for (int i = 0; i < messengers.length; ++i) {
			if (messengers[i].name().equals(destinationName)) {
				boolean success = messengers[i].sendMessage(msg, source);
				sent = true;
				
				if (verbosity > 6) {
					System.out.println("Only sending '" + msg.getFullMessage() + "' to " + 
						messengers[i] + " from " + source + 
						(success ? "; succeeded." : "; failed."));				
				}
			}
		}	
		if (!sent) {
			System.out.println("FAILED MESSAGE.  ATTEMPTED TO SEND TO UNKNOWN - " + destinationName);
			// throw an exception?
		}
	}
}