'From MIT Squeak 0.9.4 (June 1, 2003) [No updates present.] on 14 July 2011 at 10:21:29 pm'!

!FileDirectory methodsFor: 'file operations' stamp: 'CWB 6/26/2010 20:45'!
fileInContents
	"Given a directory with files named like '1-Object.st', files in all .st files in numerical order"

	| filesToFileIn filesInOrder index |

	"Grab the names of all files that look like they can be filed in"
	filesToFileIn _ self fileNamesMatching: '*.st'.

	"File in classes based in the order corresponding to the number in front of the file name
	(so that base classes are filed in before subclasses)"

	filesInOrder _ filesToFileIn asSortedCollection sortBlock: [:a :b | a asNumber < b asNumber].

	index _ 1.
	'Importing source code ...' 
		displayProgressAt: Sensor cursorPoint 
		from: 1 to: filesInOrder size
		during: [:bar |
			filesInOrder do: [:name | 
				CrLfFileStream fileIn: (self fullNameFor: name). 
				bar value: index.
				index _ index + 1]].



! !

!FileDirectory methodsFor: 'file operations' stamp: 'CWB 7/2/2010 20:21'!
fileOutClassesChangedSince: aDate

	"Goes through all classes to find ones that have changed since a given date, and files those out within the directory.  Filenames are prepended with a number representing the depth in the class heirarchy, so that they can be filed in in the correct order by 'fileInContents'"

	| allClasses classesToExport reallyOldDate index |

	reallyOldDate _ '01/01/78' asDate.

	"Get a list of all classes (but not metaclasses)"
	allClasses _ Object selectSubclasses: [ :aClass | (aClass isMeta) not ].

	"Prune the list to only those that have changed since the given date"
	index _ 1.
	'Searching for modified classes...' 
		displayProgressAt: Sensor cursorPoint
		from: 1 to: allClasses size
		during: 
			[:bar |
				classesToExport _ allClasses 
					select: [ :aClass | 
						| latestChange |
						index _ index + 1.
						bar value: index.
						latestChange _ 
							((Utilities dateOfLatestChangeToClass: aClass) ifNil: [ reallyOldDate ])
							max:
							((Utilities dateOfLatestChangeToClass: aClass class) ifNil: [ reallyOldDate ]).
						latestChange > aDate ]].
	
	index _ 1.
	'Exporting modified classes...'
		displayProgressAt: Sensor cursorPoint
		from: 1 to: classesToExport size
		during: [:bar |
			classesToExport do: [ :aClass |
				"Files are named as follows: depth-class.st 
				(where depth is a number representing how deep 
				the class is in the class heirarchy"

				| depth filename filestream |

				depth _ aClass allSuperclasses size.
				filename _ depth asString, '-', aClass name , FileDirectory dot, 'st'.
				self deleteFileNamed: filename.		"Ensure we can overwrite the file"
				filestream _ CrLfFileStream defaultToCRLF newFileNamed: (self fullNameFor: filename).

				filestream header. "timeStamp."	"Omit the timestamp!!"
				aClass sharedPools size > 0 
					ifTrue: [self fileOutSharedPoolsOn: filestream].
		
				aClass fileOutOn: filestream moveSource: false toFile: 0.
				filestream trailer; close. ]]

			

		! !

!FileDirectory methodsFor: 'file operations' stamp: 'CWB 7/14/2011 22:20'!
fileOutSpecifiedClasses: classesToExport

	"Goes through a list of classes and files them out in the same format used by fileOutClassesChangedSince:.  Useful for generating ancestral files to diff against."

	| index |

	index _ 1.

	'Exporting named classes...'
		displayProgressAt: Sensor cursorPoint
		from: 1 to: classesToExport size
		during: [:bar |
			classesToExport do: [ :filename | 
		
				| aClass classname filestream |

				"Filenames are all in format 'X-Classname.st'"
				classname _ (filename allButFirst: 2) allButLast: 3.  
				aClass _ Smalltalk at: classname asSymbol ifAbsent: [ nil ].
				aClass ifNotNil: [
					self deleteFileNamed: filename.		"Ensure we can overwrite the file"
					filestream _ CrLfFileStream defaultToCRLF newFileNamed: (self fullNameFor: filename).

					filestream header. "timeStamp."	"Omit the timestamp!!"
					"aClass sharedPools size > 0 
						ifTrue: [self fileOutSharedPoolsOn: filestream]."
		
					aClass fileOutOn: filestream moveSource: false toFile: 0.
					filestream trailer; close. 
				].
				index _ index + 1.
				bar value: index.
			].
		].

! !


!FileStream class methodsFor: 'instance creation' stamp: 'CWB 6/26/2010 21:15'!
fileIn: fullName
	"File in the entire contents of the file specified by the name provided"

	(self readOnlyFileNamed: fullName) fileIn; close.! !


!ScratchFrameMorph methodsFor: 'menu/button actions' stamp: 'CWB 7/14/2011 21:13'!
fileMenu: aMenuTitleMorph

	| menu |
	menu _ CustomMenu new.
	menu add: 'New' action: #newScratchProject.
	menu add: 'Open' action: #openScratchProject.
	menu add: 'Save' action: #saveScratchProjectNoDialog.
	menu add: 'Save As' action: #saveScratchProject.
	menu addLine.
	menu add: 'Import Project' action: #importScratchProject.
	menu add: 'Export Sprite' action: #exportSprite.
	menu addLine.
	menu add: 'Project Notes' action: #editNotes.

	Sensor shiftPressed ifTrue: [  "developer menu"
		menu addLine.
		menu add: 'Write Project Summary' action: #writeSummaryFile.
		menu add: 'Write Multiple Project Summaries' action: #writeMultipleSummaries.
		menu addLine.
		fillScreenFlag
			ifTrue: [
				menu add: 'Exit User Mode' action: #fillScreenOff]
			ifFalse: [
				menu add: 'Enter User Mode' action: #fillScreenOn.
				menu add: 'Save Image in User Mode' action: #saveImageForEndUser].
		menu addLine.
		menu add: 'Import from version control' action: #versionControlImport.
		menu add: 'Export to version control' action: #versionControlExport.
		menu add: 'Export named classes' action: #namedClassesExport.
		menu add: 'Apply latest UI skin' action: #updateInterfaceSkin ].

	menu addLine.
	menu add: 'Quit' action: #quitScratch.
	menu localize.
	
	#(2 4 5 6 7) do: [:n |
		menu labels at: n put:
			((menu labels at: n) copyFrom: 1 to: (menu labels at: n) size - 1), ScratchTranslator ellipsesSuffix].

	menu invokeOn: self at: aMenuTitleMorph bottomLeft + (0@10).

! !

!ScratchFrameMorph methodsFor: 'menu/button actions' stamp: 'CWB 7/14/2011 22:17'!
namedClassesExport

	"A kin to versionControlExport, exports a number of listed files"

	| dir files |

	dir _ self class versionControlDirectory.

	"Populate the files variable by doing an 'ls *.st' in the appropriate directory"

	files _ '1-BYOBList.st
1-BlockLabelFragment.st
1-Collection.st
1-CustomBlockDefinition.st
1-EnchantingCodeExporter.st
1-EnchantingDeviceConfiguration.st
1-EnchantingFunction.st
1-EnchantingPeripheral.st
1-FileDirectory.st
1-Morph.st
1-ObjStream.st
1-ScratchFileGetter.st
1-ScratchMedia.st
1-ScratchNotePlayer.st
1-ScratchProcess.st
1-ScratchServer.st
1-ScratchTranslator.st
1-SoundPlayer.st
1-ThumbnailCache.st
1-Utilities.st
1-VariableFrame.st
10-CArgSlotMorph.st
2-BlockMorph.st
2-Character.st
2-Form.st
2-HandMorph.st
2-LibraryItemMorph.st
2-MediaItemMorph.st
2-NXTConfiguration.st
2-NXTPeripheral.st
2-PaintCanvas.st
2-ScratchFrameMorph.st
2-ScratchPresenterMorph.st
2-ScriptableScratchMorph.st
2-SelectionMenu.st
2-StringMorph.st
3-ArgMorph.st
3-ArgumentPlaceHolderMorph.st
3-BlockLabelFragmentMorph.st
3-CommandBlockMorph.st
3-CustomMenu.st
3-DialogBoxMorph.st
3-FileList.st
3-HatBlockMorph.st
3-MultilineStringMorph.st
3-NXTDCMotor.st
3-NXTSensor.st
3-PasteUpMorph.st
3-ScratchFilePicker.st
3-ScratchListMorph.st
3-ScratchNoteSelector.st
3-ScratchPrompterMorph.st
3-ScratchSpriteMorph.st
3-ScratchStageMorph.st
3-ScrollFrameMorph.st
3-StringFieldMorph.st
3-UpdatingStringMorph.st
3-WatcherReadoutFrameMorph.st
4-BYOBListWatcherMorph.st
4-BlockEditorFrameMorph.st
4-BooleanArgMorph.st
4-CBlockMorph.st
4-ChoiceArgMorph.st
4-ColorArgMorph.st
4-ConfigurationBlockMorph.st
4-ConfigurationHatMorph.st
4-CustomCommandBlockMorph.st
4-DebuggerFrameMorph.st
4-EnchantingConfigurationDialog.st
4-EnchantingFirmwareFlashDialog.st
4-EnchantingUploadDialog.st
4-EventHatMorph.st
4-EventTitleMorph.st
4-ExpressionArgMorph.st
4-HostSystemMenusMacOSXScratch.st
4-IfElseBlockMorph.st
4-ListMultilineStringMorph.st
4-MultiArgCommandBlockMorph.st
4-MultiArgMorph.st
4-NXTButtons.st
4-NXTCompassSensor.st
4-NXTGyroSensor.st
4-NXTLightDetector.st
4-NXTMotor.st
4-NXTRangeDetector.st
4-NXTSoundSensor.st
4-NXTTouchSensor.st
4-ParagraphEditor.st
4-PluggableTextMorph.st
4-ProcedureHatBlockMorph.st
4-ReporterBlockDropTargetMorph.st
4-ReporterBlockMorph.st
4-ScratchFileChooserDialog.st
4-ScratchLibraryMorph.st
4-ScratchScriptEditorMorph.st
4-ScratchScriptsMorph.st
4-ScratchToolTipMorph.st
4-ScratchViewerMorph.st
4-SetterBlockMorph.st
4-SpecialArgMorph.st
4-SpriteArgMorph.st
4-String.st
5-AttributeArgMorph.st
5-BlockDialog.st
5-BlockLabelFragmentDialog.st
5-ChoiceArgMorphForPeripheral.st
5-CustomReporterBlockMorph.st
5-ExpressionArgMorphWithMenu.st
5-FileStream.st
5-KeyEventHatMorph.st
5-LabelFragmentBlockMorph.st
5-ListContentsBlockMorph.st
5-MouseClickEventHatMorph.st
5-MultiArgReporterBlockMorph.st
5-PredicateArgMorph.st
5-ResizableToggleButton2.st
5-ScratchCommentMorph.st
5-ToggleButton.st
5-UnevaluatedBooleanArgMorph.st
5-UnevaluatedExpressionArgMorph.st
5-VariableBlockMorph.st
5-VariableDeclarationBlockMorph.st
5-WatcherMorph.st
6-LambdaBlockMorph.st
7-ReporterScriptBlockMorph.st
8-CommandScriptBlockMorph.st
8-OSlotMorph.st
9-CReporterSlotMorph.st
9-CSlotMorph.st
9-TypeInReporterSlotMorph.st'.
	
	dir fileOutSpecifiedClasses: files lines.
	! !

!ScratchFrameMorph methodsFor: 'menu/button actions' stamp: 'CWB 7/2/2010 21:30'!
updateInterfaceSkin

	"Reads in the latest skin from the ScratchSkin folder and updates the UI appearance."

	self class readSkinFrom: (FileDirectory default directoryNamed: 'ScratchSkin').
	"self rebuildUIForNewLanguage."
	ScratchTranslator updateScratchUI.! !

!ScratchFrameMorph methodsFor: 'menu/button actions' stamp: 'CWB 6/29/2010 13:20'!
versionControlExport
	"Outputs all classes that have changed since the project start date to a directory that should be managed by an external revision control system."

	| dir date |

	dir _ self class versionControlDirectory.
	date _ self class versionControlStartDate.
	
	dir fileOutClassesChangedSince: date.	
	! !

!ScratchFrameMorph methodsFor: 'menu/button actions' stamp: 'CWB 6/29/2010 13:21'!
versionControlImport
	"Updates the classes in the Squeak VM by importing them from a 'Classes' directory that should be managed by your revision control system."

	self class versionControlDirectory fileInContents

	
! !

!ScratchFrameMorph methodsFor: 'private' stamp: 'CWB 7/2/2010 21:32'!
rebuildUIForNewLanguage
	"Rebuild my UI after the language or font has been changed."

	World fullRepaintNeeded.
	viewerPane rebuildCategorySelectors.
	self updatePanes.
	logoMorph delete.
	self createLogo.
	self
		view: scriptsPane target
		tab: scriptsPane tabPane currentTab
		category: viewerPane currentCategory.

! !


!ScratchFrameMorph class methodsFor: 'version control' stamp: 'CWB 6/29/2010 13:02'!
versionControlDirectory
	"Returns the directory used by the revision control directory to store filed-out classes."

	| baseDirectory subdirName |

	baseDirectory _ FileDirectory default.	"Use the folder the image is in"
	subdirName _ 'Classes'.

	"Create the source code revision directory is it doesn't exist"
	(baseDirectory directoryExists: subdirName) 
		ifFalse: [ baseDirectory createDirectory: subdirName ].

	"Return the code directory"
	^ baseDirectory directoryNamed: subdirName	! !

!ScratchFrameMorph class methodsFor: 'version control' stamp: 'CWB 6/29/2010 13:04'!
versionControlStartDate

	"Only classes that have changed since this date will be exported to the revision control system"

	^ '01/01/2010' asDate! !


!Utilities class methodsFor: 'miscellaneous' stamp: 'CWB 6/28/2010 21:04'!
dateOfLatestChangeToClass: aClass

	"Returns the date of when a class last changed (or nil if it was long, long ago)"
 
	| dates |

	dates _ aClass selectors asOrderedCollection 
		collect: [ :sel | 
			self dateOfLatestChangeToMethod: (aClass compiledMethodAt: sel) ].
	dates _ dates reject: [ :date | date isNil ].
	dates _ dates asSortedCollection.
	(dates isEmpty) 
		ifFalse:	[^dates last]
		ifTrue: [^nil]
! !

!Utilities class methodsFor: 'miscellaneous' stamp: 'CWB 6/28/2010 21:48'!
dateOfLatestChangeToMethod: aMethod

"Returns the date (extracted from the timestamp) for when a method was last changed, or nil if it is very old"
| datestamp |
 
datestamp _ self timeStampForMethod: aMethod.

('* */*/* *' match: datestamp)	"initials date time"
"((datestamp size > 4) and: [(datestamp occurrencesOf: Character space) = 2])"
	ifFalse: [^nil] 
	ifTrue: [^
		((datestamp allButFirst: (datestamp findDelimiters: ' ' startingAt: 1)) withoutLeadingBlanks) asDate]
! !

